@@ -8,10 +8,10 @@
 // Software.
 
 use super::register::EntryHash;
-use crate::{app::SELF_ENCRYPTION_MIN_SIZE, Error, Result, Safe};
+use crate::{Error, Result, Safe};
 use bytes::Bytes;
 use log::debug;
-use safe_network::client::{BlobAddress, SpotAddress};
+use safe_network::types::{BytesAddress, DataAddress};
 use safe_network::url::{ContentType, Scope, Url, XorUrl};
 use std::collections::BTreeSet;
 use xor_name::XorName;
@@ -54,7 +54,7 @@ impl Safe {
     /// Return the value of a Multimap on the network corresponding to the key provided
     pub async fn multimap_get_by_key(&self, url: &str, key: &[u8]) -> Result<MultimapKeyValues> {
         debug!("Getting value by key from Multimap at: {}", url);
-        let (safeurl, _) = self.parse_and_resolve_url(url).await?;
+        let safeurl = self.parse_and_resolve_url(url).await?;
 
         self.fetch_multimap_value_by_key(&safeurl, key).await
     }
@@ -66,7 +66,7 @@ impl Safe {
         hash: EntryHash,
     ) -> Result<MultimapKeyValue> {
         debug!("Getting value by hash from Multimap at: {}", url);
-        let (safeurl, _) = self.parse_and_resolve_url(url).await?;
+        let safeurl = self.parse_and_resolve_url(url).await?;
 
         self.fetch_multimap_value_by_hash(&safeurl, hash).await
     }
@@ -88,11 +88,11 @@ impl Safe {
     /// Insert a key-value pair into a Multimap on the network
     pub async fn multimap_insert(
         &self,
-        url: &str,
+        multimap_url: &str,
         entry: MultimapKeyValue,
         replace: BTreeSet<EntryHash>,
     ) -> Result<EntryHash> {
-        debug!("Inserting '{:?}' into Multimap at {}", entry, url);
+        debug!("Inserting '{:?}' into Multimap at {}", entry, multimap_url);
         let serialised_entry = rmp_serde::to_vec_named(&entry).map_err(|err| {
             Error::Serialisation(format!(
                 "Couldn't serialise the Multimap entry '{:?}': {:?}",
@@ -101,24 +101,24 @@ impl Safe {
         })?;
 
         let data = Bytes::copy_from_slice(&serialised_entry);
-        let entry_xorname = self.safe_client.store_data(data.clone(), false).await?;
-        let entry_xorurl = if data.len() < SELF_ENCRYPTION_MIN_SIZE {
-            Url::encode_spot(
-                SpotAddress::Public(entry_xorname),
-                ContentType::Raw,
-                self.xorurl_base,
-            )?
-        } else {
-            Url::encode_blob(
-                BlobAddress::Public(entry_xorname),
-                ContentType::Raw,
-                self.xorurl_base,
-            )?
-        };
+        let entry_xorname = self.safe_client.store_bytes(data.clone(), false).await?;
+        let entry_xorurl = Url::encode_bytes(
+            BytesAddress::Public(entry_xorname),
+            ContentType::Raw,
+            self.xorurl_base,
+        )?;
         let entry_ptr = Url::from_xorurl(&entry_xorurl)?;
-        let safeurl = Safe::parse_url(url)?;
-        let address = safeurl.register_address()?;
-
+        let safeurl = Safe::parse_url(multimap_url)?;
+        let address = match safeurl.address() {
+            DataAddress::Register(reg_address) => reg_address,
+            other => {
+                return Err(Error::InvalidXorUrl(format!(
+                    "The multimap url {} has an {:?} address.\
+                    To insert an entry into a multimap, the address must be a register address.",
+                    multimap_url, other
+                )))
+            }
+        };
         self.safe_client
             .write_to_register(address, entry_ptr, replace)
             .await
@@ -189,7 +189,7 @@ impl Safe {
 
 #[cfg(test)]
 mod tests {
-    use crate::{app::test_helpers::new_safe_instance, retry_loop, retry_loop_for_pattern};
+    use crate::{app::test_helpers::new_safe_instance, retry_loop_for_pattern};
     use anyhow::Result;
     use std::collections::BTreeSet;
 
@@ -201,8 +201,8 @@ mod tests {
         let xorurl_priv = safe.multimap_create(None, 25_000, true).await?;
 
         let key = b"".to_vec();
-        let received_data = retry_loop!(safe.multimap_get_by_key(&xorurl, &key));
-        let received_data_priv = retry_loop!(safe.multimap_get_by_key(&xorurl_priv, &key));
+        let received_data = safe.multimap_get_by_key(&xorurl, &key).await?;
+        let received_data_priv = safe.multimap_get_by_key(&xorurl_priv, &key).await?;
 
         assert_eq!(received_data, Default::default());
         assert_eq!(received_data_priv, Default::default());
@@ -223,8 +223,8 @@ mod tests {
         let xorurl = safe.multimap_create(None, 25_000, false).await?;
         let xorurl_priv = safe.multimap_create(None, 25_000, true).await?;
 
-        let _ = retry_loop!(safe.multimap_get_by_key(&xorurl, &key));
-        let _ = retry_loop!(safe.multimap_get_by_key(&xorurl_priv, &key));
+        let _ = safe.multimap_get_by_key(&xorurl, &key).await?;
+        let _ = safe.multimap_get_by_key(&xorurl_priv, &key).await?;
 
         let hash = safe
             .multimap_insert(&xorurl, key_val.clone(), BTreeSet::new())
@@ -285,8 +285,8 @@ mod tests {
         let xorurl = safe.multimap_create(None, 25_000, false).await?;
         let xorurl_priv = safe.multimap_create(None, 25_000, true).await?;
 
-        let _ = retry_loop!(safe.multimap_get_by_key(&xorurl, &key));
-        let _ = retry_loop!(safe.multimap_get_by_key(&xorurl_priv, &key));
+        let _ = safe.multimap_get_by_key(&xorurl, &key).await?;
+        let _ = safe.multimap_get_by_key(&xorurl_priv, &key).await?;
 
         let hash = safe
             .multimap_insert(&xorurl, key_val.clone(), BTreeSet::new())
@@ -302,14 +302,14 @@ mod tests {
             .multimap_insert(&xorurl_priv, key_val2.clone(), BTreeSet::new())
             .await?;
 
-        let received_data = retry_loop!(safe.multimap_get_by_hash(&xorurl, hash));
-        let received_data_priv = retry_loop!(safe.multimap_get_by_hash(&xorurl_priv, hash_priv));
+        let received_data = safe.multimap_get_by_hash(&xorurl, hash).await?;
+        let received_data_priv = safe.multimap_get_by_hash(&xorurl_priv, hash_priv).await?;
 
         assert_eq!(received_data, key_val.clone());
         assert_eq!(received_data_priv, key_val);
 
-        let received_data = retry_loop!(safe.multimap_get_by_hash(&xorurl, hash2));
-        let received_data_priv = retry_loop!(safe.multimap_get_by_hash(&xorurl_priv, hash_priv2));
+        let received_data = safe.multimap_get_by_hash(&xorurl, hash2).await?;
+        let received_data_priv = safe.multimap_get_by_hash(&xorurl_priv, hash_priv2).await?;
 
         assert_eq!(received_data, key_val2.clone());
         assert_eq!(received_data_priv, key_val2);
