@@ -21,7 +21,9 @@ mod constants;
 mod errors;
 
 // re-export these useful types from sn_data_types
-pub use safe_network::types::{Keypair, PublicKey, SecretKey};
+pub use safe_network::types::{
+    BytesAddress, DataAddress, Keypair, PublicKey, RegisterAddress, SafeKeyAddress, SecretKey,
+};
 
 #[cfg(feature = "app")]
 pub use app::*;
